$(document).ready(function() {
    // $("#completed_tests").dataTable({
    //     "bSearch": false,
    //     "bPaginate": false,
    //     "bLengthChange": false,
    //     "bFilter": false,
    //     "bSort": true,
    //     "bInfo": false,
    //     "bAutoWidth": false,
    //     "aaSorting": [[3, 'desc']]
    // });
});
