{% extends 'layout.jinja2.html' %}

{% block title %}Home{% endblock %}

{% block style %}
.jumbotron td {
  padding: 10px;
}
{% endblock %}

{% block body %}
      <div class="jumbotron">
        <h1>Hello!</h1>
        <p>This is an installation
        of <a href="https://github.com/datastax/cstar_perf">cstar_perf</a>,
        the Cassandra performance testing suite brought to you
        by <a href='http://www.datastax.com/'>DataStax</a>.</p>
        <p>From here you can view benchmarks run by others, or run
        your own.</p>
        <table><tr>
            <td><a class="btn btn-primary btn-lg" role="button"
                   href="/tests">View Tests</a></td>

            {% if user_is_authenticated() %}
            <td><a class="btn btn-primary
        btn-lg" role="button" href="/schedule">New Test</a></td>
            {% endif %}

        </tr></table>
        </p>
      </div>
{% endblock %}
