<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="utf-8">
        <meta http-equiv="X-UA-Compatible" content="IE=edge">
        <meta name="viewport" content="width=device-width,
                                       initial-scale=1">
        <title>cstar_perf - {% block title %}{% endblock %}</title>
         
        <link rel="stylesheet" href="/static/lib/bootstrap-3.1.1-dist/css/bootstrap.css" />
        <link rel="stylesheet" href="/static/css/app.css" />
        <link href='http://fonts.googleapis.com/css?family=Faster+One'
              rel='stylesheet' type='text/css'>
        <script src="/static/lib/jquery-2.1.0.min.js"></script>
        <script src="/static/js/app.js"></script>
        <script src="/static/js/util.js"></script>
        <script src="/static/lib/bootstrap-3.1.1-dist/js/bootstrap.js"></script>
        <script src="/static/lib/jquery.dataTables.min.js"></script>

        <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
        <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
        <!--[if lt IE 9]>
            <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
            <script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
            <![endif]-->

        <meta name="csrf_token" content="{{csrf_token()}}" />

        <meta name="google-signin-clientid" content="{{google_client_id}}" />
        <meta name="google-signin-scope" content="https://www.googleapis.com/auth/plus.profile.emails.read" />
        <meta name="google-signin-requestvisibleactions" content="http://schemas.google.com/AddActivity" />
        <meta name="google-signin-cookiepolicy" content="single_host_origin" />

    <style type="text/css">
      {% block style %}
      {% endblock %}
    </style>

    </head>
    <body>
      <div id="container">
      <nav class="navbar navbar-default" role="navigation">
        <div class="container-fluid">
          <!-- Brand and toggle get grouped for better mobile display -->
          <div class="navbar-header">
            <button type="button" class="navbar-toggle" data-toggle="collapse" data-target="#bs-example-navbar-collapse-1">
              <span class="sr-only">Toggle navigation</span>
              <span class="icon-bar"></span>
              <span class="icon-bar"></span>
              <span class="icon-bar"></span>
            </button>
            <a class="navbar-brand" href="/">cstar_perf</a>
          </div>

          <!-- Collect the nav links, forms, and other content for toggling -->
          <div class="collapse navbar-collapse" id="bs-example-navbar-collapse-1">
            <ul class="nav navbar-nav">
              <li><a href="/tests">View Tests</a></li>
              {% if user_is_authenticated() %}
              <li><a href="/schedule">New Test</a></li>
              <li id="cluster-dropdown" class="dropdown"> 
                <a href="#" class="dropdown-toggle" 
                   data-toggle="dropdown"></b>Clusters</a> 
                <ul class="dropdown-menu"> 
                  {% for c in clusters %}
                  <li><a href="/cluster/{{c}}">{{c}}</a></li>
                  {% endfor %}
                </ul> 
              </li> 
              {% endif %}
            </ul>
            <ul class="nav navbar-nav navbar-right">
              {% if user_is_authenticated() %}
                <li class="dropdown"> 
                  <a href="#" class="dropdown-toggle" 
                  data-toggle="dropdown"><b class="caret"></b> <span id="user_id">{{get_user_id()}}</span></a> 
                  <ul class="dropdown-menu"> 
                    <li><a href="/tests/user">Your Tests</a></li> 
                    <li><a href="#">Your Profile</a></li> 
                    <li class="divider"></li> 
                    <li><a href="/logout"><span class="glyphicon 
                    glyphicon-log-out"></span> Logout</a></li> 
                  </ul> 
                </li> 
                <li></li>
              {% else %}
                <li><button class="btn" id="signinButton">Sign in</button></li>
              {% endif %}
            </ul>
          </div><!-- /.navbar-collapse -->
        </div><!-- /.container-fluid -->
      </nav>



      {% block body %}
      {% endblock %}
</div>

<div class="site-footer">
  
  <hr/>
  <ul class="site-footer-links right">
    <li> <a href="http://www.datastax.com"><img src="/static/img/datastax.png"/></a></li>
  </ul>

  <ul class="site-footer-links">
    <li>© 2014 DataStax</li>
    <li><a href="https://github.com/datastax/cstar_perf">Code</a></li>
    <li><a href="https://github.com/datastax/cstar_perf/issues">Support</a></li>
  </ul>
</div>


</body>
</html>
