{% extends 'layout.jinja2.html' %}
{% from 'test_macro.jinja2.html' import test_table %}

{% block title %}Tests{% endblock %}

{% block style %}

{% endblock %}

{% block body %}

{% for cluster in clusters %}
  {% if cluster_in_progress_tests.has_key(cluster) %}
    <h2>Tests in progress in {{cluster}}:</h2>
    {{test_table(cluster_in_progress_tests[cluster])}}
  {% endif %}
{% endfor %}

{% for cluster in clusters %}
  {% if cluster_scheduled_tests.has_key(cluster) %}
    <h2>Scheduled tests in {{cluster}}:</h2>
    {{test_table(cluster_scheduled_tests[cluster])}}
  {% endif %}
{% endfor %}


<h2>Recently completed tests:</h2>
{{test_table(completed_tests, date_field='completed_date', table_id='completed_tests')}}

<script src="/static/js/tests.js"></script>
{% endblock %}
