html {
    position: relative;
    min-height: 100%;
}

body {
    margin-bottom: 100px; /*size of footer*/
}

.navbar-brand {
    font-family: 'Faster One', cursive;
    font-size: 1.8em;
}

.site-footer {
    height: 101px;
    position: absolute;
    width: 100%;
    bottom: 0;
}

.site-footer-links {
    list-style: none;
    display: block;
    margin-left: auto;
    margin-right: auto;
    width:40%;
    float:left;
    height: 50px;
    line-height: 50px;
    
}
.site-footer img {
    margin-top: -10px;
    height: 30px;
}

.site-footer-links.right, .site-footer-links.right li {
    float:right;
}


.site-footer-links li {
    display: inline-block;
    margin-left: 10px;
    margin-right: 10px;
}

.attention {
    border: 1px solid #F7ED64;
    border-radius: 0.4em;
    padding: 3px;
    background-color: rgba(255,251,198,0.5);
}

.error {
    padding: 2px 4px;
    color: #c7254e;
    background-color: #f9f2f4;
    border-radius: 4px;
    font-weight: bold;
    font-size: 1.1em;
}

.success {
    padding: 2px 4px;
    color: #598708;
    background-color: #BCF25E;
    border-radius: 4px;
    font-weight: bold;
    font-size: 1.1em;    
}