div#chart_controls {
    margin-left: 900px;
    position: absolute;
}

#chart_controls > table {
    width: 640px;
}

#chart_controls td {
    padding: 2px;
}

#chart_controls #zoom input {
    width: 50px;
}

#chart_controls table#zoom {
    padding-left: 20px;
}

#chart_controls table#zoom td {
    padding-left: 20px;
}

#zoom_drag_surface {
    fill: rgba(250, 250, 255, 0.0);
    z-index: 100;
}

svg {
  font: 10px sans-serif;
}

.axis path,
.axis line {
  fill: none;
  stroke: #000;
  shape-rendering: crispEdges;
}

.x.axis path {
  display: none;
}

.line {
  fill: none;
  stroke: steelblue;
  stroke-width: 1.5px;
}
